@@ -1,6 +1,6 @@
 #  MIT License
 #
-#  Copyright (c) 2017-2021 TileDB Inc.
+#  Copyright (c) 2017-2023 TileDB Inc.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -194,7 +194,6 @@ tiledb_ctx_set_default_tags <- function(object) {
 #' @return A JSON-formatted string with context statistics
 #' @export
 tiledb_ctx_stats <- function(object = tiledb_get_context()) {
-    stopifnot(`The 'object' variable must be a TileDB Context object` = is(object, "tiledb_ctx"),
-              `TileDB 2.4.0 or later is required` = tiledb_version(TRUE) >= "2.4.0")
+    stopifnot(`The 'object' variable must be a TileDB Context object` = is(object, "tiledb_ctx"))
     libtiledb_ctx_stats(object@ptr)
 }
